/*
 *
 *  Copyright (C) 1993-2001, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module:  imagectn
 *
 *  Author:  Andrew Hewett
 *
 *  Purpose: Image CTN Common Constants, Types, Globals and Functions
 *
 *  Last Update:      $Author: meichel $
 *  Update Date:      $Date: 2001/06/01 15:51:17 $
 *  Source File:      $Source: /share/dicom/cvs-depot/dcmtk/imagectn/apps/imagectn.h,v $
 *  CVS/RCS Revision: $Revision: 1.8 $
 *  Status:           $State: Exp $
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef IMAGECTN_H
#define IMAGECTN_H

#include "osconfig.h"    /* make sure OS specific configuration is included first */

#include "dcxfer.h"
#include "dicom.h"
#include "cond.h"
#include "assoc.h"
#include "cnf.h"
#include "ofconapp.h"

/*
 * Application Conditions
 */

#define APP_NORMAL		/* normal operation */\
	FORM_COND(FAC_APP, SEV_SUCC, 0)
#define APP_BADIMAGE		/* bad image data */\
	FORM_COND(FAC_APP, SEV_ERROR, 1)
#define APP_NOCONTEXT		/* no presentation context for SOP Class */\
	FORM_COND(FAC_APP, SEV_ERROR, 2)
#define APP_PROTOCOLERROR	/* protocol error */\
	FORM_COND(FAC_APP, SEV_ERROR, 3)
#define APP_MESSAGEFAILED	/* network message failed */\
	FORM_COND(FAC_APP, SEV_ERROR, 4)
#define APP_STOREFAILED		/* store operation failed */\
	FORM_COND(FAC_APP, SEV_ERROR, 5)
#define APP_INVALIDPEER		/* invalid peer (for move operation) */\
	FORM_COND(FAC_APP, SEV_ERROR, 6)

/*
 * Common Globals
 */

extern T_ASC_Network *net;
extern E_TransferSyntax  opt_networkTransferSyntax;
extern E_TransferSyntax  opt_writeTransferSyntax;
extern int         opt_verbose;
extern OFBool      opt_debug;
extern OFBool      opt_ignoreStoreData;
extern OFBool      opt_requireFindForMove;
extern OFBool      opt_displayImagesOnMove;
extern OFBool      opt_displayTransferProgress;
extern OFBool      opt_restrictMoveToSameVendor;
extern OFBool      opt_restrictMoveToSameHost;
extern OFBool      opt_restrictMoveToSameAE;
extern OFBool      opt_bitPreserving;
extern OFBool      opt_useMetaheader;
extern E_GrpLenEncoding  opt_groupLength;
extern E_EncodingType    opt_sequenceType;
extern E_PaddingEncoding opt_paddingType;
extern OFCmdUnsignedInt  opt_filepad;
extern OFCmdUnsignedInt  opt_itempad; 

/* tables of abstract syntaxes which image ctn supports */
extern const char* nonStorageSyntaxes[];
extern int numberOfNonStorageSyntaxes;

/*
 * Common Function Definitions
 */

extern void errmsg(const char* msg, ...);

extern OFBool peerCanDisplayImages(const char* peerName);

#endif

/*
** CVS Log
** $Log: imagectn.h,v $
** Revision 1.8  2001/06/01 15:51:17  meichel
** Updated copyright header
**
** Revision 1.7  2000/03/08 16:40:58  meichel
** Updated copyright header.
**
** Revision 1.6  1999/06/10 12:11:54  meichel
** Adapted imagectn to new command line option scheme.
**   Added support for Patient/Study Only Q/R model and C-GET (experimental).
**
** Revision 1.5  1997/08/26 14:17:19  hewett
** Added +B command line option to imagectn application.  Use of this option
** causes imagectn to bypass the dcmdata encode/decode routines when receiving
** images and write image data to disk exactly as received in a C-STORE
** command over the network.  This option does _not_ affect sending images.
**
** Revision 1.4  1997/07/21 08:59:43  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.3  1996/09/24 15:52:34  hewett
** Now uses global table of Storage SOP Class UIDs (from dcuid.h).
** Also added preliminary support for the Macintosh environment (GUSI library).
**
** Revision 1.2  1996/04/22 10:20:11  hewett
** Added global variables for restricting move destinations.
**
** Revision 1.1.1.1  1996/03/28 19:24:59  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
